#!/usr/bin/env python
#
# -------------------------------------------------------------------------------------
#
# Copyright (c) 2021 Skinnyhouse
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification, 
# are permitted provided that the following conditions are met:
#
#    - Redistributions of source code must retain the above copyright notice, this 
#    list of conditions and the following disclaimer.
#    - Redistributions in binary form must reproduce the above copyright notice, 
#    this list of conditions and the following disclaimer in the documentation and/or 
#    other materials provided with the distribution.
#    - Neither the name of the author nor the names of its contributors may be used 
#    to endorse or promote products derived from this software without specific prior 
#    written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY 
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
# SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
# BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN 
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH 
# DAMAGE.
#
# -------------------------------------------------------------------------------------
#
# This file is a basic Python plug-in for GIMP.
#
# It can be executed by selecting the menu option: 'Filters>


from gimpfu import *
#from array import array
#import math
import random

#from gtk import gdk
#w = gdk.screen_width()
#h = gdk.screen_height()

# Some global colours we may need to define

black = gimpcolor.RGB(0, 0, 0)
white = gimpcolor.RGB(255, 255, 255)
#darkgrey = gimpcolor.RGB(50, 50, 50)
#midgrey = gimpcolor.RGB(125, 125, 125)
#lightgrey = gimpcolor.RGB(170, 170, 170)
#verylightgrey = gimpcolor.RGB(222, 222, 222)
#red = gimpcolor.RGB(220,10,10)	

# Some global metallic colours we may need for creating gradients

#lightsilver = gimpcolor.RGB(204, 202, 202)
#darksilver = gimpcolor.RGB(46, 43, 43)
#lightgold = gimpcolor.RGB(250, 233, 49)
#darkgold = gimpcolor.RGB(240, 183, 42)
#lightcopper = gimpcolor.RGB(212, 125, 1)
#darkcopper = gimpcolor.RGB(177, 106, 58)
#lightbrass = gimpcolor.RGB(253, 242, 91)
#darkbrass = gimpcolor.RGB(220 , 118, 20)
#lightbronze = gimpcolor.RGB(104, 75, 33)
#darkbronze = gimpcolor.RGB(83, 69, 42)


def do_stuff(image, layer, size, number_of_glows, col_option, mode, opacity, blur, compass, do_noise, dulling, hue, sat, val, new_layer_increment, modify_kerning, distort):
    #### Main Filter Proc
	
    # Store the GIMP's settings so they can be restored
    gimp.context_push()
    # Set start undo point
    pdb.gimp_image_undo_group_start(image)	
	

    loop_ptr = 1
    first_char_layer_name = "none"
    num_layers, layer_ids = pdb.gimp_image_get_layers(image)
    loop = num_layers - loop_ptr
    layer_names_list = image.layers
    original_active_layer = image.active_layer
	
	
    visi_loop, first_char_layer  = 0, -1
    other_layers,num_char_layers = 0,0
    apply_to_custom = False
	
	
    fg_colour = pdb.gimp_context_get_foreground()
    bg_colour = pdb.gimp_context_get_background()
    current_gradient = pdb.gimp_context_get_gradient()
    current_mode = fn_set_mode(mode)
	
    blur = 500/100 * blur # adjust blur as a percentage
	
    while visi_loop < num_layers:
	    # get info about layer we are working on
        layer_name = pdb.gimp_item_get_name(layer_names_list[visi_loop])
        layer_name = layer_name.decode('utf-8')
		
		
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[visi_loop], layer_name)
			
        if test_for_custom_layer:
		
            apply_to_custom = True
			
            num_char_layers += 1
            if first_char_layer < 0: 
                first_char_layer = visi_loop
                first_char_layer_name = layer_name
                layer_names_list[visi_loop].set_offsets(0, 0)
        pdb.gimp_item_set_visible(layer_names_list[visi_loop],False) # turn off visibility
        visi_loop += 1
		
		
    while loop >= 0 and apply_to_custom:
	    # get info about layer we are working on for custom font mode
        layer_id = layer_names_list[loop]
        layer_name = pdb.gimp_item_get_name(layer_names_list[loop])
		
        #test_for_custom_layer = fn_test_for_custom_layer(image, layer_names_list[loop], layer_name)
	
        # cater for unicode characters above normal range
        display_layer_name =  pdb.gimp_item_get_name(layer_names_list[loop])
        display_layer_name = display_layer_name.decode('utf-8')	
	
        # check we have the layer id of the currently active layer
        pdb.gimp_image_set_active_layer(image, layer_id)
        layer_id = pdb.gimp_image_get_active_layer(image)
        active_layer_id = pdb.gimp_image_get_active_layer(image)


			
				
        test_for_custom_layer = fn_test_for_custom_layer(image, layer_id, layer_name)
			
        if test_for_custom_layer:

            # ensure we have the active layer and make it visible
            active_layer_id = pdb.gimp_image_get_active_layer(image)
            pdb.gimp_item_set_visible(active_layer_id, True)
            if new_layer_increment>0:
                layer_width = active_layer_id.width
                layer_height = active_layer_id.height
                #pdb.gimp_item_transform_translate(active_layer_id, new_layer_increment, 0)
                pdb.gimp_layer_resize(active_layer_id,layer_width + (new_layer_increment*2), layer_height, new_layer_increment,0 )
                if active_layer_id.width > image.width: pdb.gimp_image_resize(image, active_layer_id.width, active_layer_id.height, 0, 0)
                pdb.gimp_item_transform_translate(active_layer_id, new_layer_increment, 0)
            active_layer_id = proc_outer_glow(image, active_layer_id, size, fg_colour, bg_colour, current_gradient, col_option, current_mode, opacity, blur, compass, do_noise, dulling, hue, sat, val, new_layer_increment, modify_kerning, number_of_glows,distort)
				
				
        if (num_layers - loop_ptr) > 0: pdb.gimp_item_set_visible(active_layer_id, False)
			
        num_layers, layer_ids = pdb.gimp_image_get_layers(image)
			
        loop_ptr += 1
        loop = num_layers - loop_ptr
		
		
	
    if first_char_layer_name <> "none": # the character layer to make visible on completion
        first_char_layer_id = pdb.gimp_image_get_layer_by_name(image, first_char_layer_name)
        pdb.gimp_item_set_visible(first_char_layer_id, True)
        pdb.gimp_image_set_active_layer(image,first_char_layer_id)
    else:	
        active_layer = image.active_layer
        pdb.gimp_item_set_visible(original_active_layer, True)
        pdb.gimp_image_set_active_layer(image,original_active_layer)
	

    # Set end undo point
    pdb.gimp_image_undo_group_end(image)

    # Restore the GIMP's settings
    gimp.context_pop()
	
######################################################################################################################################



def proc_outer_glow(image, character_layer, size, fg_colour, bg_colour, current_gradient, col_option, current_mode, opacity, blur, compass, do_noise, dulling, hue, sat, val, new_layer_increment, modify_kerning, number_of_glows, distort):


    character_layer_name = pdb.gimp_item_get_name(character_layer)
    character_layer_name = character_layer_name.decode('utf-8')	
    # make a copy of the original character
    blur_level = blur
    size_level = size
		
    if modify_kerning:
        #############
        global_kerning_width = character_layer.width - (new_layer_increment * 2)
        # attempt to set global kerning vectors for all unkerned characters
        global_kerning_name = character_layer_name[0]+".png"
        #pdb.gimp_message(global_kerning_name + " = " + str(pdb.gimp_image_get_vectors_by_name(image, global_kerning_name) ) )
        if pdb.gimp_image_get_vectors_by_name(image, global_kerning_name)<>None:
            #pdb.gimp_message(global_kerning_name + " found...")
            global_kerning_id = pdb.gimp_image_get_vectors_by_name(image, global_kerning_name)
            pdb.gimp_image_lower_item_to_bottom(image, global_kerning_id)
        else:
            #pdb.gimp_message(global_kerning_name + " not found... as " + str(pdb.gimp_image_get_vectors_by_name(image, global_kerning_name)) )
            #global_kerning_width = width - right_margin - blur_amount
            global_kerning_id = pdb.gimp_vectors_new(image, global_kerning_name)
            pdb.gimp_image_insert_vectors(image, global_kerning_id, None, -1)
            pdb.gimp_image_lower_item_to_bottom(image, global_kerning_id)
            points1 = [0,character_layer.height/2] * 3
            points2 = [global_kerning_width,character_layer.height/2] * 3
            points = points1 + points2
            #pdb.gimp_message(points)
            global_kerning_id = pdb.gimp_vectors_stroke_new_from_points(global_kerning_id ,0, len(points), points, False)
			
			
            ############# Not implemented - not required
            # attempt to modify any existing kerning paths
            # get paths/vectors ids
            num_kerning_paths, kerning_vector_ids = pdb.gimp_image_get_vectors(image)
            kerning_list = image.vectors
				
            kerning_path_loop = 0
            #while kerning_path_loop < num_kerning_paths:
            #    # look for recognised kerning path
            #    kerning_path_name = pdb.gimp_item_get_name(kerning_list[kerning_path_loop])
            #    kerning_path_name = kerning_path_name.decode('utf-8')
            #        
            #    if (len(kerning_path_name) == 5 and ".png" in kerning_path_name) or (len(kerning_path_name) == 6 and ".png" in kerning_path_name):
            #        kerning_path_adjustment = 0
            #        if kerning_path_name.startswith(character_layer_name[0]):
            #            kerning_path_adjustment = 0
            #            if new_layer_increment<=0:
            #                kerning_path_adjustment = abs((new_layer_increment))
            #                #kerning_path_adjustment = abs(char_x_offset)
            #            else:
            #                kerning_path_adjustment = 0-(abs((new_layer_increment)))
            #                #kerning_path_adjustment = 0-(abs(char_x_offset))
            #                #pdb.gimp_message("kerning_path_adjustment = " + str(kerning_path_adjustment) )
            #        if kerning_path_adjustment<>0:
            #            kerning_vector = kerning_list[kerning_path_loop]
            #            #kerning_vector = kerning_vector_ids[kerning_path_loop]
            #            num_strokes, stroke_ids = pdb.gimp_vectors_get_strokes(kerning_vector)                             
            #            type, num_points, controlpoints, closed  = pdb.gimp_vectors_stroke_get_points(kerning_vector, stroke_ids[0])
            #            kerning_target_width = controlpoints[6] + kerning_path_adjustment
            #            v_position = pdb.gimp_image_get_vectors_position(image, kerning_vector)
            #            pdb.gimp_image_remove_vectors(image, kerning_vector)
            #            new_vector_id = pdb.gimp_vectors_new(image, kerning_path_name)
            #            pdb.gimp_image_insert_vectors(image, new_vector_id, None, v_position)
            #            points1 = [controlpoints[0],controlpoints[1]] * 3
            #            points2 = [kerning_target_width,controlpoints[7]] * 3
            #            points = points1 + points2
            #            #pdb.gimp_message(points)
            #            new_vector_id = pdb.gimp_vectors_stroke_new_from_points(new_vector_id ,0, len(points), points, False)
            #            #pdb.gimp_image_lower_item_to_bottom(image, kerning_vector)
            #            #pdb.gimp_item_set_name(new_vector_id, pdb.gimp_item_get_name(kerning_list[kerning_path_loop]))
            #                
			#		
            #    kerning_path_loop += 1
			
	
    blur_loop = number_of_glows
    blur_factor = float(blur_level/number_of_glows)
    #while blur_level > 0.1:
    while blur_loop > 0:
	
	
        layer_pos = pdb.gimp_image_get_item_position(image, character_layer)
        copy_layer = pdb.gimp_layer_copy(character_layer, True)
        image.add_layer(copy_layer, layer_pos)
        #pdb.gimp_image_insert_layer(image, copy_layer, None, layer_pos)
        pdb.gimp_image_lower_item(image, copy_layer)
        pdb.gimp_item_set_name(copy_layer,"Copy Layer")
        if col_option<>3: pdb.gimp_drawable_edit_clear(copy_layer) # need to retain copy for this option

        pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
        #pdb.gimp_selection_shrink(image,1) ###
        if size_level>0: pdb.gimp_selection_grow(image,size_level) ###
		
        ##### Experimental trying to and add some variation to the glow
        if distort:
            random.seed()
            threshold = random.randrange(80,201,10)
            spread = random.randrange(4,25,1)
            smooth = random.randrange(1,4,1)
            pdb.script_fu_distress_selection(image, character_layer, threshold, spread, 4, smooth, True, True)

        pdb.gimp_selection_feather(image,3)
		
        if col_option == 0 or col_option == 1 or col_option == 2: 
            if col_option == 2:
                pdb.gimp_drawable_edit_fill(copy_layer, 4) # fill character outline
            else:
                pdb.gimp_drawable_edit_fill(copy_layer, col_option) # fill character outline
        elif col_option == 4:
            blend_x0, blend_y0, blend_x1, blend_y1 = fn_do_blend(image, character_layer, compass, size)
			
            pdb.gimp_image_select_item(image, 2, character_layer) # alpha select
            pdb.gimp_drawable_edit_gradient_fill(copy_layer, 0, 0, False, 1, 0, True, blend_x0, blend_y0, blend_x1, blend_y1)
	
            #pdb.gimp_selection_shrink(image, size) # shrink not needed
			
            #selection, x1, x2, y1, y2, s_width, s_height = fn_selection_info(image)
			
			
        pdb.gimp_selection_none(image)
		
        pdb.plug_in_gauss(image, copy_layer, blur_level, blur_level, 0)
        #if do_noise: pdb.plug_in_hsv_noise(image, copy_layer, dulling, hue, sat, val)
        if blur_loop<number_of_glows: copy_layer = pdb.gimp_image_merge_down(image, copy_layer, 1)
        #character_layer = pdb.gimp_image_merge_down(image, character_layer, 1)

        blur_loop -= 1
        #blur_level = blur_level*0.25
        size_level = size_level * 0.25
        blur_level = blur_level - blur_factor
        #size_level = size_level - blur_factor
	
		
		
    if do_noise: pdb.plug_in_hsv_noise(image, copy_layer, dulling, hue, sat, val)
    
    pdb.gimp_layer_set_opacity(copy_layer, opacity)
    #if blur_loop>0: copy_layer = pdb.gimp_image_merge_down(image, copy_layer, 1)
    character_layer = pdb.gimp_image_merge_down(image, character_layer, 1)
    pdb.gimp_item_set_name(character_layer,character_layer_name)
	
    #dummy = crashnow
    return character_layer
	
	
######################################################################################################################################

def fn_do_blend(image, layer, compass, size):

    width = layer.width
    copy_layer = pdb.gimp_layer_copy(layer, TRUE)
    temp = pdb.gimp_image_insert_layer(image, copy_layer, None, -1)
    pdb.gimp_selection_none(image)
    pdb.gimp_image_set_active_layer(image, copy_layer)
    pdb.plug_in_autocrop_layer(image, copy_layer)
    pdb.gimp_drawable_edit_fill(copy_layer, 0) # fill character outline
    pdb.gimp_image_select_item(image, 2, copy_layer)
    # get co-ordinate values for blending
    current_selection, cs_x1, cs_y1, cs_x2, cs_y2  = pdb.gimp_selection_bounds(image)	
	
    blend_x0, blend_y0, blend_x1, blend_y1 = fn_gradient_direction(compass, width, cs_x1-size, cs_y1-size, cs_x2+size, cs_y2+size)
    pdb.gimp_selection_none(image)
    image.remove_layer(copy_layer)
    #gradient_mode = fn_set_mode(mode)

    return blend_x0, blend_y0, blend_x1, blend_y1
	
	
######################################################################################################################################

def fn_gradient_direction(compass, width, cs_x1, cs_y1, cs_x2, cs_y2):

 
    if compass == 0:
        # North
        blend_x0 = width/2
        blend_y0 = cs_y1
        blend_x1 = width/2
        blend_y1 = cs_y2
    elif compass == 1:
        # North East
        blend_x0 = width
        blend_y0 = cs_y1
        blend_x1 = 0
        blend_y1 = cs_y2
    elif compass == 2:
        # East
        blend_x0 = width
        blend_y0 = cs_y1 + ((cs_y2-cs_y1)/2)
        blend_x1 = 0
        blend_y1 = cs_y1 + ((cs_y2-cs_y1)/2)
    elif compass == 3:
        # South East
        blend_x0 = width
        blend_y0 = cs_y2
        blend_x1 = 0
        blend_y1 = cs_y1
    elif compass == 4:
        # South
        blend_x0 = width/2
        blend_y0 = cs_y2
        blend_x1 = width/2
        blend_y1 = cs_y1
    elif compass == 5:
        # South West
        blend_x0 = 0
        blend_y0 = cs_y2
        blend_x1 = width
        blend_y1 = cs_y1
    elif compass == 6:
        # West
        blend_x0 = 0
        blend_y0 = ((cs_y2-cs_y1)/2)
        blend_x1 = width
        blend_y1 = ((cs_y2-cs_y1)/2)
    else:
        # North West
        blend_x0 = cs_x1
        blend_y0 = cs_y1
        blend_x1 = cs_x2
        blend_y1 = cs_y2
		
    return blend_x0, blend_y0, blend_x1, blend_y1
	
######################################################################################################################################
######################################################################################################################################

def fn_test_for_custom_layer(img,l_id,l_name):
    # checks that the current layer is compatible with custom fonts
    test = False
    if len(l_name) == 1:
        test = True
    elif len(l_name) == 5 or len(l_name) == 6:
        
        if ".png" in l_name:
            test = True
            if len(l_name) == 6:
                part_name_val = unicode(l_name[0:2])
                if part_name_val <= 65535: test = False
    # select the contents of the layer
    pdb.gimp_image_select_item(img,2,l_id)
    # Test to see if the selection actually contains something
    sel_is_empty = pdb.gimp_selection_is_empty(img)
    # If there are no pixels to work with flag the layer as not viable to work on
    if sel_is_empty: test = False
    # Deselect
    pdb.gimp_selection_none(img)
    return test
	
######################################################################################################################################

######################################################################################################################################
	
def fn_set_mode(mode_ptr):
    # Use New Modes
    new_mode = mode_ptr
    temp_mode = 47
    if new_mode == 0:
        temp_mode = 0
        n_mode_info = "Normal"
    elif new_mode == 1:
        temp_mode = 1
        n_mode_info = "Dissolve"
    elif new_mode == 2:
        temp_mode = 57
        _mode_info = "Color Erase"
    elif new_mode == 3:
        temp_mode = 59
        n_mode_info = "Merge"
    elif new_mode == 4:
        temp_mode = 60
        n_mode_info = "Split"
    elif new_mode == 5:
        temp_mode = 36
        n_mode_info = "Lighten Only"
    elif new_mode == 6:
        temp_mode = 55
        n_mode_info = "Luma Luminance Lighten"
    elif new_mode == 7:
        temp_mode = 31
        n_mode_info = "Screen"
    elif new_mode == 8:
        temp_mode = 42
        n_mode_info = "Dodge"
    elif new_mode == 9:
        temp_mode = 33
        n_mode_info = "Addition"
    elif new_mode == 10:
        temp_mode = 35
        n_mode_info = "Darken only"
    elif new_mode == 11:
        temp_mode = 54
        n_mode_info = "Luma Luminance darken"
    elif new_mode == 12:
        temp_mode = 30
        n_mode_info = "Multiply"
    elif new_mode == 13:
        temp_mode = 43
        n_mode_info = "Burn"
    elif new_mode == 14:
        temp_mode = 53
        n_mode_info = "Linear burn"
    elif new_mode == 15:
        temp_mode = 23
        n_mode_info = "Overlay"
    elif new_mode == 16:
        temp_mode = 45
        n_mode_info = "Soft light"
    elif new_mode == 17:
        temp_mode = 44
        n_mode_info = "Hard light"
    elif new_mode == 18:
        temp_mode = 48
        n_mode_info = "Vivid light"
    elif new_mode == 19:
        temp_mode = 49
        n_mode_info = "Pin light"
    elif new_mode == 20:
        temp_mode = 50
        n_mode_info = "Linear light"
    elif new_mode == 21:
        temp_mode = 51
        n_mode_info = "Hard mix"
    elif new_mode == 22:
        temp_mode = 32
        n_mode_info = "Difference"
    elif new_mode == 23:
        temp_mode = 52
        n_mode_info = "Exclusion"
    elif new_mode == 24:
        temp_mode = 34
        n_mode_info = "Subtract"
    elif new_mode == 25:
        temp_mode = 46
        n_mode_info = "Grain extract"
    elif new_mode == 26:
        temp_mode = 47
        n_mode_info = "Grain merge"
    elif new_mode == 27:
        temp_mode = 41
        n_mode_info = "Divide"
    elif new_mode == 28:
        temp_mode = 37
        n_mode_info = "Hue HSV"
    elif new_mode == 29:
        temp_mode = 38
        n_mode_info = "Saturation HSV"
    elif new_mode == 30:
        temp_mode = 39
        n_mode_info = "Color HSL"
    elif new_mode == 31:
        temp_mode = 40
        n_mode_info = "Value HSL"
    elif new_mode == 32:
        temp_mode = 24
        n_mode_info = "Hue LCH"
    elif new_mode == 33:
        temp_mode = 25
        n_mode_info = "Chroma LCH"
    elif new_mode == 34:
        temp_mode = 26
        n_mode_info = "Color LCH"
    elif new_mode == 35:
        temp_mode = 27
        n_mode_info = "Lightness LCH"
    elif new_mode == 36:
        temp_mode = 56
        n_mode_info = "Luminance"
		
		
    mode = temp_mode
    return mode




######################################################################################################################################
    #### Useful Functions
# -------------------------------------------------------------------------------------
	
def fn_selection_info(im):
    # returns selection info
    selection,x1,y1,x2,y2 = pdb.gimp_selection_bounds(im)
    s_width, s_height = x2 - x1, y2 - y1
	
    return selection, x1, x2, y1, y2, s_width, s_height


######################################################################################################################################
	
def fn_selection_amount(sel_value,r_ness,adjustment):
    # 
    value = (math.sqrt(sel_value)*r_ness)*adjustment
    return value

######################################################################################################################################

def fn_set_metal_colour(mc):
    # Returns mode for metallic colours and the colour name
    if (mc == 1):
        n_colour = "Gold"
        pdb.gimp_context_set_foreground(lightgold)
        pdb.gimp_context_set_background(darkgold)
        c_mode = 19#SOFTLIGHT
    elif (mc == 2):
        n_colour = "Copper"
        pdb.gimp_context_set_foreground(darkcopper)
        pdb.gimp_context_set_background(lightcopper)
        c_mode = 21#GRAIN MERGE
    elif (mc == 3):
        n_colour = "Brass"
        pdb.gimp_context_set_foreground(darkbrass)
        pdb.gimp_context_set_background(lightbrass)
        c_mode = 19#SOFTLIGHT
    elif (mc == 4):
        n_colour = "Bronze"
        pdb.gimp_context_set_foreground(darkbronze)
        pdb.gimp_context_set_background(lightbronze)
        c_mode = 19#SOFTLIGHT
    else:
        n_colour = "Silver"
        pdb.gimp_context_set_foreground(lightgrey)
        pdb.gimp_context_set_background(midgrey)
        c_mode = 19#SOFTLIGHT

    return c_mode, n_colour

######################################################################################################################################
	
def proc_messagebox(message):
    old_handler = pdb.gimp_message_get_handler()
    pdb.gimp_message_set_handler(2)
    pdb.gimp_message(str(message))
    pdb.gimp_message_set_handler(old_handler)
	
######################################################################################################################################
		
def proc_trim_away(img, layer, amount):
    #### Trim off pixels from a layer
    # set our feather settings
    our_feather = True
    our_feather_radius = 2
    #get feather settings
    feather = pdb.gimp_context_get_feather()
    feather_x, feather_y = pdb.gimp_context_get_feather_radius()
    # make sure that feather is set
    if not feather: pdb.gimp_context_set_feather(our_feather)
    # use our feather radius
    pdb.gimp_context_set_feather_radius(our_feather_radius, our_feather_radius)
    # select the contents of the layer
    pdb.gimp_image_select_item(img, 2, layer)
    # shrink the selection
    pdb.gimp_selection_shrink(img, amount)
    # invert selection
    pdb.gimp_selection_invert(img)
    # trim away pixels
    successful = pdb.gimp_edit_cut(layer)
    # deselect
    pdb.gimp_selection_none(img)
    # reset feather settings
    pdb.gimp_context_set_feather(feather)
    pdb.gimp_context_set_feather_radius(feather_x, feather_y)
	
    #### End of Proc_trim_away
	
######################################################################################################################################
	
def proc_text_bevel(image, text_layer, bevel_width, bevel_depth, feather, black, white, midgrey, bevel_mode):
    #### Create a bevel around text and return the new layer as bevel_layer
    #Rem Select text outline and save it
    pdb.gimp_image_select_item(image, 2, text_layer)
    saved_selection = pdb.gimp_selection_save(image)

    #### Rem Make a bumpmap layer ######
    bumpmap_layer = pdb.gimp_layer_new_from_drawable(text_layer, image)
    pdb.gimp_image_insert_layer(image, bumpmap_layer, None, -1)
    pdb.gimp_item_set_name(bumpmap_layer, "Text Layer Bumpmap")
    pdb.gimp_image_raise_item_to_top(image, bumpmap_layer)

    pdb.gimp_selection_feather(image, feather)
    blurred_selection = pdb.gimp_selection_save(image)
    loop = bevel_width
    pdb.gimp_selection_shrink(image, loop)

    while (pdb.gimp_selection_is_empty(image) == True ):
        loop = loop - 1
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop)
        
    bevel_width = min(bevel_width, loop)
    pdb.gimp_image_select_item(image, 2, blurred_selection)

    pdb.gimp_context_set_foreground(black)
    pdb.gimp_drawable_fill(bumpmap_layer, 0)

    loop = 1
    #Rem Temp dirty fill_value 
    fill_value = (255 / bevel_width)
    while (loop <= bevel_width):

        #Rem Using Temp fill_value until something better comes to mind
        pdb.gimp_image_select_item(image, 2, blurred_selection)
        pdb.gimp_selection_shrink(image, loop -1)
        greycolour = int(fill_value)
        grey = gimpcolor.RGB(greycolour, greycolour, greycolour)
        pdb.gimp_context_set_foreground(grey)

        pdb.gimp_edit_fill(bumpmap_layer, 0)

        #Rem Temp fill_value
        fill_value = fill_value + (255 / bevel_width)
        loop = loop + 1

    pdb.gimp_context_set_foreground(white)
    pdb.gimp_image_select_item(image, 2, blurred_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == FALSE):
        pdb.gimp_edit_fill(bumpmap_layer, 0)

    pdb.gimp_selection_none(image)

    #### Make Bevel from bumpmap ####
    bevel_layer = pdb.gimp_layer_new_from_drawable(text_layer, image)
    pdb.gimp_image_insert_layer(image, bevel_layer, None, -1)
    pdb.gimp_item_set_name(bevel_layer, "Bevel")
    pdb.gimp_image_raise_item_to_top(image, bevel_layer)

    pdb.gimp_context_set_foreground(midgrey)
    pdb.gimp_drawable_fill(bevel_layer, 0)
	
    pdb.plug_in_bump_map(image, bevel_layer, bumpmap_layer, 135, 35, bevel_depth, 0, 0, 0, 0, True, False, 0)
    pdb.gimp_layer_set_mode(bevel_layer, bevel_mode)
    pdb.gimp_layer_set_opacity(bevel_layer, 80)
	
    #### Delete area outside the bevel ####
    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_invert(image)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)

    pdb.gimp_image_select_item(image, 2, saved_selection)
    pdb.gimp_selection_shrink(image, bevel_width)

    if (pdb.gimp_selection_is_empty(image) == False):
        pdb.gimp_edit_clear(bevel_layer)
	
    pdb.gimp_image_select_item(image, 2, saved_selection)
 
    pdb.gimp_selection_none(image)
    pdb.gimp_drawable_set_visible(bumpmap_layer, False)	
    pdb.gimp_image_remove_channel(image, saved_selection)
    pdb.gimp_image_remove_channel(image, blurred_selection)
    pdb.gimp_image_remove_layer(image, bumpmap_layer)
    pdb.gimp_image_raise_item_to_top(image, bevel_layer)
    pdb.gimp_item_set_linked(bevel_layer, True)
    return bevel_layer
	
#### End of proc_text_bevel

######################################################################################################################################
	
register(
    "python_fu_Custom_Font_Outer_Glow",
    "Provides options for adding an outer glow effect to custom font characters",
    "Provides options for adding an outer glow effect to custom font characters",
    "Skinnyhouse",
    "Open source (BSD 3-clause license)",
    "2021",
    "<Image>/Filters/Custom Font Tools/Custom Font Outer Glow...",
    "RGB*",
    [
    (PF_SPINNER, "amount", "Initial glow size: ", 0, (0, 100, 1)),
    (PF_SPINNER, "detail", "Number of glow iterations: ", 4, (1, 10, 1)),
    (PF_OPTION, "colour_option", "Glow colour options", 0, ("Use current foreground colour", "Use current background colour", "Use current pattern", "Use a copy of the character", "Use the current gradient")),
    (PF_OPTION, "mode_option2", "New Mode \nto use with font face:", 0, ("Normal", "Dissolve","Color erase", "Merge", "Split", "Lighten only", "Luma/Luminance lighten only", "Screen", "Dodge", "Addition", "Darken only", "Luma/Luminance darken only", "Multiply", "Burn", "Linear burn", "Overlay", "Soft light", "Hard light", "Vivid light", "Pin light", "Linear light", "Hard mix", "Difference", "Exclusion", "Subtract", "Grain extract", "Grain merge", "Divide", "Hue (HSV)", "Saturation (HSV)", "Color (HSL)", "Value (HSL)", "Hue (LCH)", "Chroma (LCH)", "Color (LCH)", "Lightness (LCH)", "Luminance")),
    (PF_SPINNER, "opacity", "Glow opacity: ", 100, (1, 100, 1)),
    (PF_SPINNER, "G_blur", "Amount of blur to use (as %): ", 4.0, (0.5, 100.0, 0.5)),
    (PF_OPTION, "pf_compass", "Gradient blend direction: ", 7,("N","NE","E","SE","S","SW","W","NW")),
    (PF_TOGGLE, "pf_add_hsv_noise", "Add HSV noise: ", 0),
    (PF_SPINNER, "pf_hsv_dullness", "Dulling: ", 1,(1,8,1)),
    (PF_SPINNER, "pf_hsv_hue", "Hue: ", 50,(1,180,1)),
    (PF_SPINNER, "pf_hsv_sat", "Saturation: ", 50,(0,255,1)),
    (PF_SPINNER, "pf_hsv_val", "Value: ", 50,(0,255,1)),
    (PF_SPINNER, "inc_layer", "Increase character layer size: ", 0, (0, 200, 1)),
    (PF_TOGGLE, "pf_attempt_kerning", "Attempt to modify kerning: ", 0),
    (PF_TOGGLE, "pf_distress", "Distress blur: ", 0),
    ],
    [],
    do_stuff)

main()
